/*
 * ===========================
 * VDK Visual Development Kit
 * odbc subsystem
 * Version 0.1
 * Revision 0.0
 * January 2001
 * ===========================
 *
 * Copyright (C) 2001, Mario Motta
 * Developed by Pierre-Louis Malatray <pierrelouis.malatray@free.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#ifndef _vdkodbcquery_h
#define _vdkodbcquery_h

#include <vdkodbc/vdkodbcconnect.h>

typedef struct {
	SQLSMALLINT num;
	SQLCHAR		name[VDKODBC_MAXCOLNAMELEN+1];
	SQLSMALLINT	namelen;
	SQLSMALLINT	type;
	SQLUINTEGER	size;
	SQLSMALLINT	dec;
	SQLSMALLINT	nullable;
} ColDesc;

/*!
\class VDKODBCQuery
\brief Provides querying facilities to a configured unixODBC datasource
*/
class VDKODBCQuery : public VDKODBCObject
{
	private:
		//! Statement handle
		SQLHSTMT			Hstmt;
		//! VDKODBCConnection object pointer
		VDKODBCConnection	*pConn;
		//! VDKODBCQuery object name
		VDKString			name;

	protected:
		/*!
		Free statement handle
		*/
		void FreeStmt();
		/*!
		Tests connection pointer validity
		\return true if pointer is valid, false otherwise
		*/
		bool ConnSanity();
		/*!
		Get a column descriptor
		\param colname Column name
		\param coldesc Adress of a column descriptor structure
		*/
		int GetColDesc(const char *colname, ColDesc& coldesc);
		/*!
		Get a column descriptor
		\param colname Column name
		\param coldesc Adress of a column descriptor structure
		*/
		int GetColDesc(VDKString colname, ColDesc& coldesc);
		/*!
		Get a column descriptor
		\param colnum Column number
		\param coldesc Adress of a column descriptor structure
		*/
		int GetColDesc(unsigned int colnum, ColDesc& coldesc);
		/*!
		Get data
		\param numcol Column number
		\param coltype Column type
		\param buffer Buffer to store data
		\param buflen Buffer length
		*/
		int GetData(unsigned int numcol, int coltype, char *buffer, int buflen);

	public:
		/*!
		Constructor
		\param query_name Name for the object
		\param conn Pointer to a VDKODBCConnection object
		*/
		VDKODBCQuery(const char *query_name, VDKODBCConnection *conn);
		/*!
		Constructor
		\param query_name Name for the object
		\param conn Pointer to a VDKODBCConnection object
		*/
		VDKODBCQuery(VDKString query_name, VDKODBCConnection *conn);
		/*!
		Destructor
		*/
		~VDKODBCQuery();
		/*!
		Returns the object name
		*/
		VDKString Name() {
			return name;
		};
		/*!
		Prepare the SQL query
		\return
		VDKODBC_INVALID_CONN_POINTER if the connection object pointer is NULL or \n
		an ODBC error number in case of error. \n
		SQL_SUCCESS otherwise.
		\param query Query to prepare
		\param ... Optional parameter list (printf capability)
		*/
		int Prepare(const char *query, ...);
		/*!
		Prepare the SQL query
		\return
		VDKODBC_INVALID_CONN_POINTER if the connection object pointer is NULL or \n
		an ODBC error number in case of error. \n
		SQL_SUCCESS otherwise.
		\param query Query to prepare
		*/
		int Prepare(VDKString query);
		/*!
		Execute the prepared SQL query.
		\return
		An ODBC error number in case of error, SQL_SUCCESS otherwise.
		*/
		int Exec();
		/*!
		Execute the SQL query by preparing and executing it.
		\return
		An ODBC error number in case of error, SQL_SUCCESS otherwise.
		\param query Query to execute
		\param ... Optional parameter list (printf capability)
		*/
		int ExecDirect(const char *query, ...);
		/*!
		Execute the SQL query by preparing and executing it.
		\return
		An ODBC error number in case of error, SQL_SUCCESS otherwise.
		\param query Query to execute
		*/
		int ExecDirect(VDKString query);
		/*!
		Returns select query concerned columns
		*/
		int ColumnCount();
		/*!
		Returns select query concerned rows
		*/
		int RowCount();
		/*!
		Fetch select query concerned data (one record)
		\return
		An ODBC error number in case of error, SQL_SUCCESS otherwise.
		*/
		int Fetch();
		/*!
		Closes current select statement.
		This seems to be necessary for some ODBC drivers
		*/
		void Close();
		/*!
		Returns true if current select statement cursor points on last record
		false otherwise
		*/
		bool Eof();
		/*!
		Returns a column name
		\return
		An ODBC error number in case of error, SQL_SUCCESS otherwise.
		\param colnum Column number (starting at 1)
		\warning May return an isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetColumnName(unsigned int colnum);
		/*!
		Get a field value in text format
		\return
		An ODBC error number in case of error, SQL_SUCCESS otherwise.
		\param colname Column name
		\warning May return an isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetFieldValue(const char *colname);
		/*!
		Get a field value in text format
		\return
		An ODBC error number in case of error, SQL_SUCCESS otherwise.
		\param colname Column name
		\warning May return an isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetFieldValue(VDKString colname);
		/*!
		Get a field value in text format
		\return
		An ODBC error number in case of error, SQL_SUCCESS otherwise.
		\param colnum Column number
		\warning May return an isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetFieldValue(unsigned int colnum);
		/*!
		Returns SQL column data type
		\param colname Column name
		\return SQL_UNKNOWN_TYPE on error
		else an SQL data type corresponding to SQL data type as following : \n
		- SQL_CHAR => CHAR(n) \n
		- SQL_VARCHAR => VARCHAR(n) \n
		- SQL_LONGVARCHAR => LONG VARCHAR \n
		- SQL_WCHAR => WCHAR(n) \n
		- SQL_WVARCHAR => VARWCHAR(n) \n
		- SQL_WLONGVARCHAR => LONGWVARCHAR \n
		- SQL_DECIMAL => DECIMAL(p,s) \n
		- SQL_NUMERIC => NUMERIC(p,s) \n
		- SQL_SMALLINT => SMALLINT \n
		- SQL_INTEGER => INTEGER \n
		- SQL_REAL => REAL \n
		- SQL_FLOAT => FLOAT(p) \n
		- SQL_DOUBLE => DOUBLE PRECISION \n
		- SQL_BIT => BIT \n
		- SQL_TINYINT => TINYINT \n
		- SQL_BIGINT => BIGINT \n
		- SQL_BINARY => BINARY(n) \n
		- SQL_VARBINARY => VARBINARY(n) \n
		- SQL_LONGVARBINARY => LONG VARBINARY \n
		- SQL_DATE => DATE (ODBC 2) \n
		- SQL_TIME => TIME(p) (ODBC 2) \n
		- SQL_TIMESTAMP => TIMESTAMP(p) (ODBC 2) \n
		- SQL_TYPE_DATE => DATE (ODBC 3) \n
		- SQL_TYPE_TIME => TIME(p) (ODBC 3) \n
		- SQL_TYPE_TIMESTAMP => TIMESTAMP(p) (ODBC 3) \n
		- SQL_INTERVAL_MONTH => INTERVAL MONTH(p) \n
		- SQL_INTERVAL_YEAR => INTERVAL YEAR(p) \n
		- SQL_INTERVAL_YEAR_TO_MONTH => INTERVAL YEAR(p) TO MONTH \n
		- SQL_INTERVAL_DAY => INTERVAL DAY(p) \n
		- SQL_INTERVAL_HOUR => INTERVAL HOUR(p) \n
		- SQL_INTERVAL_MINUTE => INTERVAL MINUTE(p) \n
		- SQL_INTERVAL_SECOND => INTERVAL SECOND(p,q) \n
		- SQL_INTERVAL_DAY_TO_HOUR => INTERVAL DAY(p) TO HOUR \n
		- SQL_INTERVAL_DAY_TO_MINUTE => INTERVAL DAY(p) TO MINUTE \n
		- SQL_INTERVAL_DAY_TO_SECOND => INTERVAL DAY(p) TO SECOND(q) \n
		- SQL_INTERVAL_HOUR_TO_MINUTE => INTERVAL HOUR(p) TO MINUTE \n
		- SQL_INTERVAL_HOUR_TO_SECOND => INTERVAL HOUR(p) TO SECOND(q) \n
		- SQL_INTERVAL_MINUTE_TO_SECOND => INTERVAL MINUTE(p) TO SECOND(q) \n
		- SQL_GUID => GUID
		*/
		int GetColumnType(const char *colname);
		/*!
		Returns SQL column data type
		\param colname Column name
		\return SQL_UNKNOWN_TYPE on error
		else an SQL data type
		see int GetColumnType(const char *colname) for details
		*/
		int GetColumnType(VDKString colname);
		/*!
		Returns SQL column data type
		\param colnum Column number
		\return SQL_UNKNOWN_TYPE on error
		else an SQL data type
		see int GetColumnType(const char *colname) for details
		*/
		int GetColumnType(unsigned int colnum);
		/*!
		Returns the size of the column on the data source.
		\param colname Column name
		\return If the column size cannot be determined, returns 0.
		*/
		int GetColumnSize(const char *colname);
		/*!
		Returns the size of the column on the data source.
		\param colname Column name
		\return If the column size cannot be determined, returns 0.
		*/
		int GetColumnSize(VDKString colname);
		/*!
		Returns the size of the column on the data source.
		\param colnum Column number
		\return If the column size cannot be determined, returns 0.
		*/
		int GetColumnSize(unsigned int colnum);
		/*!
		Returns the number of decimal digits of the column on the data source.
		\param colname Column name
		\return If the number of decimal digits cannot be determined or is not applicable, the driver returns 0.
		*/
		int GetColumnDecDigits(const char *colname);
		/*!
		Returns the number of decimal digits of the column on the data source.
		\param colname Column name
		\return If the number of decimal digits cannot be determined or is not applicable, the driver returns 0.
		*/
		int GetColumnDecDigits(VDKString colname);
		/*!
		Returns the number of decimal digits of the column on the data source.
		\param colnum Column number
		\return If the number of decimal digits cannot be determined or is not applicable, the driver returns 0.
		*/
		int GetColumnDecDigits(unsigned int colnum);
		/*!
		Return a value that indicates whether the column allows NULL values.
		\param colname Column name
		\return
		- SQL_NO_NULLS: The column does not allow NULL values.
		- SQL_NULLABLE: The column allows NULL values.
		- SQL_NULLABLE_UNKNOWN: Cannot determine if the column allows NULL values.
		*/
		int GetColumnNullable(const char *colname);
		/*!
		Return a value that indicates whether the column allows NULL values.
		\param colname Column name
		\return 
		- SQL_NO_NULLS: The column does not allow NULL values.
		- SQL_NULLABLE: The column allows NULL values.
		- SQL_NULLABLE_UNKNOWN: Cannot determine if the column allows NULL values.
		*/
		int GetColumnNullable(VDKString colname);
		/*!
		Return a value that indicates whether the column allows NULL values.
		\param colnum Column number
		\return 
		- SQL_NO_NULLS: The column does not allow NULL values.
		- SQL_NULLABLE: The column allows NULL values.
		- SQL_NULLABLE_UNKNOWN: Cannot determine if the column allows NULL values.
		*/
		int GetColumnNullable(unsigned int colnum);
		/*!
		Returns column type name
		\param sqldatatype SQL data type (coming from GetColumnType)
		\return May return isNull() string
		\warning Not reliable for now since ODBC drivers are not completely
		ODBC 3 compliant. Use with care.
		*/
		VDKString GetColumnTypeName(int sqldatatype);

};

#endif


