/*
 * ===========================
 * VDK Visual Development Kit
 * odbc subsystem
 * Version 0.1
 * Revision 0.0
 * January 2001
 * ===========================
 *
 * Copyright (C) 2001, Mario Motta
 * Developed by Pierre-Louis Malatray <pierrelouis.malatray@free.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#ifndef _vdkodbctable_h
#define _vdkodbctable_h

#include <vdk/dlist.h>
#include <vdkodbc/vdkodbc.h>
#include <vdkodbc/vdkodbcbase.h>
#include <vdkodbc/vdkodbccolumn.h>
#include <vdkodbc/vdkodbcpriv.h>

// Privilege list
typedef VDKList<VDKODBCPriv>			PrivList;
typedef VDKListIterator<VDKODBCPriv>	PrivListIterator;
// Column list
typedef VDKList<VDKODBCColumn>			ColumnList;
typedef VDKListIterator<VDKODBCColumn>	ColumnListIterator;

//! ODBC resultset positions for SQLColumns
#define VDKODBC_SQLCOLUMNS_NAME				4
#define VDKODBC_SQLCOLUMNS_DATA_TYPE		5
#define VDKODBC_SQLCOLUMNS_TYPE				6
#define VDKODBC_SQLCOLUMNS_SIZE				7
#define VDKODBC_SQLCOLUMNS_BUFLENGTH		8
#define VDKODBC_SQLCOLUMNS_DECDIGITS		9
#define VDKODBC_SQLCOLUMNS_PRECISION		10
#define VDKODBC_SQLCOLUMNS_NULLABLE			11
#define VDKODBC_SQLCOLUMNS_REMARKS			12
#define VDKODBC_SQLCOLUMNS_DEFAULT			13
#define VDKODBC_SQLCOLUMNS_SQL_DATA_TYPE	14
#define VDKODBC_SQLCOLUMNS_DATETIME_SUB		15
#define VDKODBC_SQLCOLUMNS_BYTE_LENGTH		16
#define VDKODBC_SQLCOLUMNS_ORDPOS			17
#define VDKODBC_SQLCOLUMNS_ISNULLABLE		18
//! ODBC resultset positions for SQLTablePrivileges
#define VDKODBC_SQLTABLEPRIV_GRANTOR		4
#define VDKODBC_SQLTABLEPRIV_GRANTEE		5
#define VDKODBC_SQLTABLEPRIV_PRIV			6
#define VDKODBC_SQLTABLEPRIV_GRANTABLE		7

/*!
\class VDKODBCTable
\brief Contains one table structure with attributes and privileges
*/

class VDKODBCTable : public VDKODBCBase
{
	private:
		//! Table catalog name
		VDKString	Catalog;
		//! Table schema name
		VDKString	Schema;
		//! Contains table privileges
		PrivList	Privileges;
		//! Contains table columns
		ColumnList	Columns;
		/*!
		Returns colnum column pointer
		\param colnum Column number ( >= 0 and < GetNbColumns())
		\return NULL if column not found
		*/
		VDKODBCColumn* GetPColumn(unsigned int colnum);
		/*!
		Returns colname column pointer
		\param colname Column name (case sensitive)
		\return NULL if column not found
		*/
		VDKODBCColumn* GetPColumn(VDKString colname);
	public:
		/*!
		Constructor
		*/
		VDKODBCTable() {};
		/*!
		Destructor
		*/
		~VDKODBCTable() {};
		/*!
		Returns catalog string
		Catalog name; isNull() if not applicable to the data source.
		If a driver supports catalogs for some tables but not for others,
		such as when the driver retrieves data from different DBMSs,
		it returns an empty string ("") for those tables that do not have catalogs
		\warning May return isNull() string
		*/
		VDKString GetCatalog() {
			return Catalog;
		};
		/*!
		Returns schema string
		Schema name; isNull() if not applicable to the data source.
		If a driver supports schemas for some tables but not for others,
		such as when the driver retrieves data from different DBMSs,
		it returns an empty string ("") for those tables that do not have schemas.
		\warning May return isNull() string
		*/
		VDKString GetSchema() {
			return Schema;
		};
		/*!
		Set attribute value
		\param index Which attribute to set
		\param value New value
		Possible values for index :
		- VDKODBC_NAME		object name
		- VDKODBC_TYPENAME	object type name
		- VDKODBC_REMARKS	remarks
		- VDKODBC_TABLE_CAT	table catalog
		- VDKODBC_TABLE_SCH table schema
		For internal use only.
		*/
		virtual void SetStrValue(int index, const char *value);
		/*!
		Load Columns attributes
		*/
		int LoadColumnsAttr(SQLHENV henv, SQLHDBC hconn);
		/*!
		Returns table columns quantity
		*/
		unsigned int GetNbColumns();
		/*!
		Returns a column name
		\param colnum Column number ( >= 0 and < GetNbColumns())
		\warning May return isNull() string
		*/
		VDKString GetColumnName(unsigned int colnum);
		/*!
		Returns column number from column name or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colname Column name
		\warning First column is number 0
		*/
		int GetColumnNum(VDKString colname);
		/*!
		Returns column type name
		\param colnum Column number ( >= 0 and < GetNbColumns())
		\warning May return isNull() string
		*/
		VDKString GetColumnTypeName(unsigned int colnum);
		/*!
		Returns column type name
		\param colname Column name
		\warning May return isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetColumnTypeName(VDKString colname);
		/*!
		Returns column remarks
		\param colnum Column number ( >= 0 and < GetNbColumns())
		\warning May return isNull() string
		*/
		VDKString GetColumnRemarks(unsigned int colnum);
		/*!
		Returns column remarks
		\param colname Column name
		\warning May return isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetColumnRemarks(VDKString colname);
		/*!
		Returns column size or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colnum Column number ( >= 0 and < GetNbColumns())
		\warning result is type and driver dependent 
		*/
		int GetColumnSize(unsigned int colnum);
		/*!
		Returns column size or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colname Column name
		\warning result is type and driver dependent 
		call GetLastError() to get the error definition string.
		*/
		int GetColumnSize(VDKString colname);
		/*!
		Returns column buffer length to contain data or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colnum Column number ( >= 0 and < GetNbColumns())
		*/
		int GetColumnBufLength(unsigned int colnum);
		/*!
		Returns column buffer length to contain data  or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colname Column name
		call GetLastError() to get the error definition string.
		*/
		int GetColumnBufLength(VDKString colname);
		/*!
		Returns column decimal digits or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colnum Column number ( >= 0 and < GetNbColumns())
		*/
		int GetColumnDecDigits(unsigned int colnum);
		/*!
		Returns column decimal digits or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colname Column name
		call GetLastError() to get the error definition string.
		*/
		int GetColumnDecDigits(VDKString colname);
		/*!
		Returns column radix or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colnum Column number ( >= 0 and < GetNbColumns())
		*/
		int GetColumnRadix(unsigned int colnum);
		/*!
		Returns column radix or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colname Column name
		call GetLastError() to get the error definition string.
		*/
		int GetColumnRadix(VDKString colname);
		/*!
		Returns column default value
		\param colnum Column number ( >= 0 and < GetNbColumns())
		\warning May return isNull() string
		*/
		VDKString GetColumnDefaultValue(unsigned int colnum);
		/*!
		Returns column default value
		\param colname Column name
		\warning May return isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetColumnDefaultValue(VDKString colname);
		/*!
		Returns column datetime sub or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colnum Column number ( >= 0 and < GetNbColumns())
		*/
		int GetColumnDateTimeSub(unsigned int colnum);
		/*!
		Returns column datetime sub or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colname Column name
		call GetLastError() to get the error definition string.
		*/
		int GetColumnDateTimeSub(VDKString colname);
		/*!
		Returns column ordinal position or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colnum Column number ( >= 0 and < GetNbColumns())
		*/
		int GetColumnOrdinalPos(unsigned int colnum);
		/*!
		Returns column ordinal position or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colname Column name
		call GetLastError() to get the error definition string.
		*/
		int GetColumnOrdinalPos(VDKString colname);
		/*!
		Returns column nullability or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colnum Column number ( >= 0 and < GetNbColumns())
		\return VDKODBC_NULLABLE or VDKODBC_NOT_NULLABLE if no error occur.
		*/
		int GetColumnNullable(unsigned int colnum);
		/*!
		Returns column nullability or
		VDKODBC_ERR_COLUMN_NOT_FOUND if column not found
		\param colname Column name
		call GetLastError() to get the error definition string.
		\return VDKODBC_NULLABLE or VDKODBC_NOT_NULLABLE if no error occur.
		*/
		int GetColumnNullable(VDKString colname);
		/*!
		Returns column nullability string
		\param colnum Column number ( >= 0 and < GetNbColumns())
		\warning May return isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetColumnIsNullable(unsigned int colnum);
		/*!
		Returns column nullability string
		\param colname Column name
		\warning May return isNull() string
		call GetLastError() to get the error definition string.
		*/
		VDKString GetColumnIsNullable(VDKString colname);
		/*!
		Load table privileges
		*/
		int LoadPrivileges(SQLHENV henv, SQLHDBC hconn);
};

#endif
