/*
 * ===========================
 * VDK Visual Development Kit
 * odbc subsystem
 * Version 0.1
 * Revision 0.0
 * January 2001
 * ===========================
 *
 * Copyright (C) 2001, Mario Motta
 * Developed by Pierre-Louis Malatray <pierrelouis.malatray@free.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */ 
#include <vdkodbc/vdkodbcconnect.h>

/*
 */
VDKODBCConnection::VDKODBCConnection()
{
	HEnv				= SQL_NULL_HENV;
	HConn				= SQL_NULL_HDBC;
	TimeOut				= 10;
	connected			= false;
}

/*
 */
VDKODBCConnection::~VDKODBCConnection()
{
	// Have to free all handles needed for connection
	// Hope that all will be OK
	Disconnect();
}

/*
 */
int
VDKODBCConnection::AllocEnv()
{
	// Already allocated, don't alloc twice
	if (HEnv != SQL_NULL_HENV) return SQL_SUCCESS;
	// Alloc environnement handle
	if (!SQLTest(SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &HEnv), HEnv, HConn))
		return LastNumError;
	// Set ODBC version
	if (!SQLTest(SQLSetEnvAttr(HEnv, SQL_ATTR_ODBC_VERSION, (void*)SQL_OV_ODBC3, 0), HEnv, HConn)) {
		FreeEnv();
		return LastNumError;
	}
	return SQL_SUCCESS;
}

/*
 */
void
VDKODBCConnection::FreeEnv()
{
	if (HEnv != SQL_NULL_HENV) {
		SQLFreeHandle(SQL_HANDLE_ENV, HEnv);
		HEnv = SQL_NULL_HENV;
	}
}

/*
 */
int
VDKODBCConnection::AllocConn()
{
	// Already allocated, don't alloc twice
	if (HConn != SQL_NULL_HDBC) return SQL_SUCCESS;
	// Alloc connection handle
	if (!SQLTest(SQLAllocHandle(SQL_HANDLE_DBC, HEnv, &HConn), HEnv, HConn)) {
		FreeEnv();
		return LastNumError;
	}
	return SQL_SUCCESS;
}

/*
 */
void
VDKODBCConnection::FreeConn()
{
	if (HConn != SQL_NULL_HDBC) {
		SQLFreeHandle(SQL_HANDLE_DBC, HConn);
		HConn = SQL_NULL_HDBC;
	}
}

/*
 */
int
VDKODBCConnection::PrepareConnection()
{
	if (Connected()) return SQL_SUCCESS;
	// Environment handle
	if (!SQL_OK(AllocEnv())) return LastNumError;
	// Connection handle
	if (!SQL_OK(AllocConn())) {
		FreeEnv();
		return LastNumError;
	}
	unsigned int timeout = GetTimeOut();
	// Connection attributes
	if (!SQLTest(SQLSetConnectAttr(HConn, SQL_LOGIN_TIMEOUT, (SQLPOINTER)&timeout, 0), HEnv, HConn)) {
		FreeConnection();
		return LastNumError;
	}
	return SQL_SUCCESS;
}

/*
 */
void
VDKODBCConnection::FreeConnection()
{
	FreeConn();
	FreeEnv();
}

void
VDKODBCConnection::SetConnected(bool newvalue)
{
	connected = newvalue;
	if (connected) {
		// Now we are connected !!
		//
		// Should be interesting to generate an event here
		//
	} else {
		// Now we are disconnected !!
		//
		// Should be interesting to generate an event here
		//
	}
}

/*
 */
int
VDKODBCConnection::Connect(VDKString Datasource, VDKString UserName,
						   VDKString Pwd)
{
	if (Connected()) return SQL_SUCCESS;
	if (!SQL_OK(PrepareConnection())) return LastNumError;
	// Connection
	if (!SQLTest(SQLConnect(HConn, (SQLCHAR*) Datasource.c_str(), SQL_NTS,
							(SQLCHAR*) UserName.c_str(), SQL_NTS,
							(SQLCHAR*) Pwd.c_str(), SQL_NTS), HEnv, HConn)) {
		FreeConnection();
		return LastNumError;
	}
	SetConnected(true);
	DataSource = Datasource;
	// Now we are connected !!
	//
	// Should be interesting to generate an event here
	//
	// Automatically gets all table names and attributes
	// No because it can take a long time and be useless
	//if (!SQL_OK(GetTables())) return LastNumError;
	return SQL_SUCCESS;
}

/*
 */
int
VDKODBCConnection::Disconnect()
{
	if (!Connected()) return SQL_SUCCESS;
	// Have to free all handles needed for connection
	// Disconnect from the datasource
	if (!SQLTest(SQLDisconnect(HConn))) return LastNumError;
	// Free odbc and env handles
	FreeConnection();
	SetConnected(false);
	DataSource = "";
	// Delete all table descriptors
	TableListIterator it(Tables);
	for (; it; it++) delete it.current();
	Tables.flush();
	return SQL_SUCCESS;
}

/*
 */
VDKODBCTable*
VDKODBCConnection::GetPTable(unsigned int tablenum)
{
	if (tablenum >= GetNbTables()) {
		Err(VDKODBC_ERR_TABLE_NOT_FOUND,
			"VDKODBCConnection::GetPTable, table number : %d",
			tablenum);
		return NULL;
	}
	return Tables[tablenum];
}

/*
 */
VDKODBCTable*
VDKODBCConnection::GetPTable(VDKString tablename)
{
	TableListIterator it(Tables);
	for (; it; it++) {
		if (it.current()->GetName() == tablename)
			return it.current();
	}
	Err(VDKODBC_ERR_TABLE_NOT_FOUND,
		"VDKODBCConnection::GetPTable, table name : %s",
		tablename.c_str());
	return NULL;
}

/*
 */
int
VDKODBCConnection::GetTableNum(VDKString tablename)
{
	int rslt = 0;
	TableListIterator it(Tables);
	for (; it; it++) {
		if (it.current()->GetName() == tablename)
			return rslt;
		rslt++;
	}
	return VDKODBC_ERR_TABLE_NOT_FOUND;
}

/*
 */
int
VDKODBCConnection::GetDataSources(VDKString& Result, int user_or_system_or_both)
{
	int				len = SQL_MAX_DSN_LENGTH+1;
	SQLUSMALLINT	direction;
	SQLCHAR			servername[len];
	SQLSMALLINT		srvlength;
	SQLCHAR			description[len];
	SQLSMALLINT		desclength;
	SQLRETURN		rslt;
	bool			freenv = false;
	
	Result = "";
	// HEnv allocation if necessary
	if (HEnv == SQL_NULL_HENV) {
		if (!SQLTest(AllocEnv(), HEnv, HConn)) return LastNumError;
		freenv = true;
	}
	switch(user_or_system_or_both) {
		case VDKODBC_FETCH_USER:	// Look for user DSNs
			direction = SQL_FETCH_FIRST_USER;
			break;
		case VDKODBC_FETCH_SYSTEM:	// Look for system DSNs
			direction = SQL_FETCH_FIRST_SYSTEM;
			break;
		default:					// Look for all DSNs
			direction = SQL_FETCH_FIRST;
	}
	do {
		rslt = SQLDataSources(HEnv, direction, servername, len, &srvlength,
					          description, len, &desclength);
		if (rslt != SQL_NO_DATA) Result.Concatf("%s\t%s\n", servername, description);
		direction = SQL_FETCH_NEXT;
	} while (SQL_OK(rslt));
	if (rslt != SQL_NO_DATA) {
		SQLTest(rslt, HEnv, HConn);
	}
	else rslt = SQL_SUCCESS;
	if (freenv) FreeEnv();
	return rslt;
}

/*
 */
int
VDKODBCConnection::GetDrivers(VDKString& Result)
{
	SQLUSMALLINT	direction = SQL_FETCH_FIRST;
	SQLCHAR			description[VDKODBC_MAX_BUF];
	SQLSMALLINT		desclength;
	SQLCHAR			drvattributes[VDKODBC_MAX_BUF];
	SQLSMALLINT		drvattrlength;
	SQLRETURN		rslt;
	bool			freenv = false;
	
	Result = "";
	// HEnv allocation if necessary
	if (HEnv == SQL_NULL_HENV) {
		if (!SQLTest(AllocEnv(), HEnv, HConn)) return LastNumError;
		freenv = true;
	}
	do {
		rslt = SQLDrivers(HEnv, direction, description, VDKODBC_MAX_BUF, &desclength,
							drvattributes, VDKODBC_MAX_BUF, &drvattrlength);
		for (int i = 0; i < drvattrlength; i++) {
			if ((drvattributes[i] == 0) && (drvattributes[i+1] != 0))
				drvattributes[i] = '\t';
		}
		if (rslt != SQL_NO_DATA) Result.Concatf("%s\t%s\n", description, drvattributes);
		direction = SQL_FETCH_NEXT;
	} while (SQL_OK(rslt));
	if (rslt != SQL_NO_DATA) {
		SQLTest(rslt, HEnv, HConn);
	}
	else rslt = SQL_SUCCESS;
	if (freenv) FreeEnv();
	return rslt;
}

/*
 */
int
VDKODBCConnection::LoadTablesAttr()
{
	SQLHSTMT	Hstmt;
	SQLRETURN	rslt;
	SQLCHAR		buffer[VDKODBC_MAXCOLNAMELEN] = "";
	SQLINTEGER	len;
	
	if (!Connected()) {
		Err(VDKODBC_ERR_CONN_NEEDED, "VDKODBCConnection::LoadTablesAttr");
		return LastNumError;
	}
	// Delete all table descriptors
	TableListIterator it(Tables);
	for (; it; it++) delete it.current();
	Tables.flush();
	// Allocate a statement handle
	if (!SQLTest(SQLAllocHandle(SQL_HANDLE_STMT, HConn, &Hstmt), HEnv, HConn)) return LastNumError;
/*	Not supported yet by the driver manager plm 20/01/2001
	// Set SQL_ATTR_METADATA_ID to false
	if (!SQLTest(SQLSetStmtAttr(Hstmt, SQL_ATTR_METADATA_ID, (void*)SQL_FALSE, 0), HEnv, HConn)) {
		SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
		return LastNumError;
	}
*/
	// Try getting tables
	if (!SQLTest(SQLTables(Hstmt, NULL, 0, NULL, 0, NULL, 0, NULL, 0), HEnv, HConn)) {
		SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
		return LastNumError;
	}
	// Fetch the data
	rslt = SQLFetch(Hstmt);  
	while ((rslt != SQL_NO_DATA) && SQL_OK(rslt)) {
		// Create a VDKODBCTable object
		VDKODBCTable *table = new VDKODBCTable();
		if (!table) {
			Err(VDKODBC_ERR_MEMORY, "VDKODBCConnection::LoadTablesAttr");
			SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
			return LastNumError;
		}
		for (int i = VDKODBC_SQLTABLES_CATALOG; i <= VDKODBC_SQLTABLES_REMARKS; i++) {
			if (!SQLTest(SQLGetData(Hstmt, i, SQL_C_CHAR, buffer, VDKODBC_MAXCOLNAMELEN, &len), HEnv, HConn)) {
				SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
				return LastNumError;
			}
			switch (i) {
				case VDKODBC_SQLTABLES_CATALOG:
					table->SetStrValue(VDKODBC_TABLE_CAT, (const char*)buffer);
					break;
				case VDKODBC_SQLTABLES_SCHEMA:
					table->SetStrValue(VDKODBC_TABLE_SCH, (const char*)buffer);
					break;
				case VDKODBC_SQLTABLES_NAME:
					table->SetStrValue(VDKODBC_NAME, (const char*)buffer);
					break;
				case VDKODBC_SQLTABLES_TYPE:
					table->SetStrValue(VDKODBC_TYPENAME, (const char*)buffer);
					break;
				case VDKODBC_SQLTABLES_REMARKS:
					table->SetStrValue(VDKODBC_REMARKS, (const char*)buffer);
					break;
			}
			buffer[0] = 0;
		}
		// Add the pointer to the list
		Tables.add(table);
		rslt = SQLFetch(Hstmt);
	}
	// Free statement handle
	SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
	return SQL_SUCCESS;
}

/*
 */
VDKString
VDKODBCConnection::GetTableName(unsigned int tablenum)
{
	VDKString Result;

	if (tablenum >= this->GetNbTables()) return Result;
	Result = Tables[tablenum]->GetName();
	return Result;
}

/*
 */
VDKString
VDKODBCConnection::GetTableType(unsigned int tablenum)
{
	VDKString Result;

	if (tablenum >= GetNbTables()) return Result;
	Result = Tables[tablenum]->GetTypeName();
	return Result;
}

/*
 */
VDKString
VDKODBCConnection::GetTableType(VDKString tablename)
{
	VDKString Result;

	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return Result;
	Result = table->GetTypeName();
	return Result;
}

/*
 */
VDKString
VDKODBCConnection::GetTableRemarks(unsigned int tablenum)
{
	VDKString Result;

	if (tablenum >= GetNbTables()) return Result;
	Result = Tables[tablenum]->GetRemarks();
	return Result;
}

/*
 */
VDKString
VDKODBCConnection::GetTableRemarks(VDKString tablename)
{
	VDKString Result;

	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return Result;
	Result = table->GetRemarks();
	return Result;
}

/*
 */
VDKString
VDKODBCConnection::GetTableCatalog(unsigned int tablenum)
{
	VDKString Result;

	if (tablenum >= GetNbTables()) return Result;
	Result = Tables[tablenum]->GetCatalog();
	return Result;
}

/*
 */
VDKString
VDKODBCConnection::GetTableCatalog(VDKString tablename)
{
	VDKString Result;

	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return Result;
	Result = table->GetCatalog();
	return Result;
}

/*
 */
VDKString
VDKODBCConnection::GetTableSchema(unsigned int tablenum)
{
	VDKString Result;

	if (tablenum >= GetNbTables()) return Result;
	Result = Tables[tablenum]->GetSchema();
	return Result;
}

/*
 */
VDKString
VDKODBCConnection::GetTableSchema(VDKString tablename)
{
	VDKString Result;

	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return Result;
	Result = table->GetSchema();
	return Result;
}

/*
 */
unsigned int
VDKODBCConnection::GetNbTables()
{
	return Tables.size();
}

/*
 */
int
VDKODBCConnection::LoadColumnsAttr(VDKString tablename)
{
	if (!Connected()) {
		Err(VDKODBC_ERR_CONN_NEEDED,
			"VDKODBCConnection::LoadColumnsAttr, table name %s",
			tablename.c_str());
		return LastNumError;
	}
	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return VDKODBC_ERR_TABLE_NOT_FOUND;
	if (!SQL_OK(table->LoadColumnsAttr(HEnv, HConn))) {
		LastNumError = table->GetLastNumError();
		LastError = table->GetLastError();
		return LastNumError;
	}
	return SQL_SUCCESS;
}

/*
 */
unsigned int
VDKODBCConnection::GetNbColumns(VDKString tablename)
{
	VDKODBCTable* table = GetPTable(tablename);
	if (!table) return VDKODBC_ERR_TABLE_NOT_FOUND;
	return table->GetNbColumns();
}

/*
 */
VDKString
VDKODBCConnection::GetColumnName(VDKString tablename, unsigned int colnum)
{
	VDKString Result;

	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return Result;
	Result = table->GetColumnName(colnum);
	return Result;
}

/*
 */
VDKString
VDKODBCConnection::GetColumnTypeName(VDKString tablename, VDKString colname)
{
	VDKString Result;

	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return Result;
	Result = table->GetColumnTypeName(colname);
	return Result;
}

/*
 */
VDKString
VDKODBCConnection::GetColumnRemarks(VDKString tablename, VDKString colname)
{
	VDKString Result;

	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return Result;
	Result = table->GetColumnRemarks(colname);
	return Result;
}

/*
 */
int
VDKODBCConnection::GetColumnSize(VDKString tablename, VDKString colname)
{
	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return VDKODBC_ERR_TABLE_NOT_FOUND;
	return table->GetColumnSize(colname);
}

/*
 */
int
VDKODBCConnection::GetColumnBufLength(VDKString tablename, VDKString colname)
{
	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return VDKODBC_ERR_TABLE_NOT_FOUND;
	return table->GetColumnBufLength(colname);
}

/*
 */
int
VDKODBCConnection::GetColumnDecDigits(VDKString tablename, VDKString colname)
{
	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return VDKODBC_ERR_TABLE_NOT_FOUND;
	return table->GetColumnDecDigits(colname);
}

/*
 */
int
VDKODBCConnection::GetColumnRadix(VDKString tablename, VDKString colname)
{
	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return VDKODBC_ERR_TABLE_NOT_FOUND;
	return table->GetColumnRadix(colname);
}

/*
 */
VDKString
VDKODBCConnection::GetColumnDefaultValue(VDKString tablename, VDKString colname)
{
	VDKString Result;

	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return Result;
	Result = table->GetColumnDefaultValue(colname);
	return Result;
}

/*
 */
int
VDKODBCConnection::GetColumnDateTimeSub(VDKString tablename, VDKString colname)
{
	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return VDKODBC_ERR_TABLE_NOT_FOUND;
	return table->GetColumnDateTimeSub(colname);
}

/*
 */
int
VDKODBCConnection::GetColumnOrdinalPos(VDKString tablename, VDKString colname)
{
	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return VDKODBC_ERR_TABLE_NOT_FOUND;
	return table->GetColumnOrdinalPos(colname);
}

/*
 */
int
VDKODBCConnection::GetColumnNullable(VDKString tablename, VDKString colname)
{
	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return VDKODBC_ERR_TABLE_NOT_FOUND;
	return table->GetColumnNullable(colname);
}

/*
 */
VDKString
VDKODBCConnection::GetColumnIsNullable(VDKString tablename, VDKString colname)
{
	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return VDKString("");
	return table->GetColumnIsNullable(colname);
}

/*
 */
int
VDKODBCConnection::LoadTablePrivileges(VDKString tablename)
{
	if (!Connected()) {
		Err(VDKODBC_ERR_CONN_NEEDED,
			"VDKODBCConnection::LoadTablePrivileges, table name %s",
			tablename.c_str());
		return LastNumError;
	}
	VDKODBCTable *table = GetPTable(tablename);
	if (!table) return VDKODBC_ERR_TABLE_NOT_FOUND;
	if (!SQL_OK(table->LoadPrivileges(HEnv, HConn))) {
		LastNumError = table->GetLastNumError();
		LastError = table->GetLastError();
		return LastNumError;
	}
	return SQL_SUCCESS;
}

//==========================================================================================
// Ci-dessous les mthodes  revoir et  complter
//==========================================================================================

/*
Cette mthode est abandonne pour l'instant 16/01/2001
Elle sense afficher une fentre mais le handle demand n'est pas compatible  priori
 */
int
VDKODBCConnection::DriverConnect(VDKString ConnectionString, void *WindowHandle,
								 unsigned short Completion)
{
/*	
	SQLCHAR		OutString[VDKODBC_MAX_BUF];
	SQLSMALLINT	retsize;
	if (Connected()) return SQL_SUCCESS;
	if (!SQL_OK(PrepareConnection())) return LastNumError;
	if (!SQLTest(SQLDriverConnect(HConn, WindowHandle, (SQLCHAR*)ConnectionString.c_str(),
								ConnectionString.size(), OutString, VDKODBC_MAX_BUF,
								&retsize, Completion))) return LastNumError;
	ConnectionString = (const char*)OutString;
*/
	return SQL_SUCCESS;
}

/*
 */
int
VDKODBCConnection::BrowseConnect(VDKString ConnectionString)
{
	SQLCHAR		OutString[VDKODBC_BUF] = "";
	SQLCHAR		InString[VDKODBC_BUF] = "";
	SQLSMALLINT	retsize = 0;
	SQLRETURN	rslt;
	
	if (Connected()) return SQL_SUCCESS;
	if (!SQL_OK(PrepareConnection())) return LastNumError;
	strncpy((char*)InString, ConnectionString.c_str(), VDKODBC_BUF - 1);
	do {
		rslt = SQLBrowseConnect(HConn, InString, strlen((const char*)InString), OutString, VDKODBC_BUF, &retsize);
		if (rslt == SQL_NEED_DATA) GetUserInput(OutString, InString);
	} while (rslt == SQL_NEED_DATA);
	if (SQLTest(rslt, HEnv, HConn)) {
		if (retsize > 0) {
			ConnectionString = (const char*)OutString;
			// Connection
			SetConnected(true);
		} else {
			ConnectionString = "";
			FreeConnection();
		}
		rslt = SQL_SUCCESS;
	} else FreeConnection();
	return rslt;
}

/*
 */
int
VDKODBCConnection::GetUserInput(SQLCHAR *InStr, SQLCHAR *OutStr)
{
	return SQL_SUCCESS;
}

/*
Cette mthode est abandonne pour l'instant 16/01/2001
SQLGetInfo devrait tre divise en fonctions plus spcialises
 */
int
VDKODBCConnection::GetInfo(unsigned short infotype, void *answer, short bufferlen)
{
	SQLSMALLINT		stringlen = 0;
/*
	if ((infotype != SQL_ODBC_VER) && !Connected()) {
		Err(VDKODBC_ERR_CONN_NEEDED, "VDKODBCConnection::GetInfo");
		return LastNumError;
	}
*/
	if (!SQLTest(SQLGetInfo(HConn, infotype, answer, bufferlen, &stringlen)))
		return LastNumError;
	return SQL_SUCCESS;
}

/*
 */
bool
VDKODBCConnection::FuncExists(unsigned short funcID)
{
	SQLUSMALLINT	exist;

	SQLGetFunctions(HConn, funcID, &exist);
	return exist;
}

/*
 */
VDKString
VDKODBCConnection::GetDrvFunc()
{
	VDKString		Result;

	Result = "SQL_API_SQLALLOCHANDLE\t";
	FuncExists(SQL_API_SQLALLOCHANDLE) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLBINDCOL\t";
	FuncExists(SQL_API_SQLBINDCOL) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLCANCEL\t";
	FuncExists(SQL_API_SQLCANCEL) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLCLOSECURSOR\t";
	FuncExists(SQL_API_SQLCLOSECURSOR) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLCOLATTRIBUTE\t";
	FuncExists(SQL_API_SQLCOLATTRIBUTE) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLCONNECT\t";
	FuncExists(SQL_API_SQLCONNECT) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLCOPYDESC\t";
	FuncExists(SQL_API_SQLCOPYDESC) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLDATASOURCES\t";
	FuncExists(SQL_API_SQLDATASOURCES) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLDESCRIBECOL\t";
	FuncExists(SQL_API_SQLDESCRIBECOL) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLDISCONNECT\t";
	FuncExists(SQL_API_SQLDISCONNECT) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLDRIVERS\t";
	FuncExists(SQL_API_SQLDRIVERS) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLENDTRAN\t";
	FuncExists(SQL_API_SQLENDTRAN) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLEXECDIRECT\t";
	FuncExists(SQL_API_SQLEXECDIRECT) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLEXECUTE\t";
	FuncExists(SQL_API_SQLEXECUTE) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLFETCH\t";
	FuncExists(SQL_API_SQLFETCH) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLFETCHSCROLL\t";
	FuncExists(SQL_API_SQLFETCHSCROLL) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLFREEHANDLE\t";
	FuncExists(SQL_API_SQLFREEHANDLE) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLFREESTMT\t";
	FuncExists(SQL_API_SQLFREESTMT) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLGETCONNECTATTR\t";
	FuncExists(SQL_API_SQLGETCONNECTATTR) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLGETCURSORNAME\t";
	FuncExists(SQL_API_SQLGETCURSORNAME) ? Result += "Yes\n" : Result += "No\n";
	Result += "SQL_API_SQLGETDATA\t";
	FuncExists(SQL_API_SQLGETDATA) ? Result += "Yes\n" : Result += "No\n";
	// To complete
	return Result;
}

