/*
 * ===========================
 * VDK Visual Development Kit
 * odbc subsystem
 * Version 0.1
 * Revision 0.0
 * January 2001
 * ===========================
 *
 * Copyright (C) 2001, Mario Motta
 * Developed by Pierre-Louis Malatray <pierrelouis.malatray@free.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */ 
#include <vdkodbc/vdkodbc.h>
#include <vdkodbc/vdkodbcquery.h>
/*
 */
VDKODBCQuery::VDKODBCQuery(const char *query_name, VDKODBCConnection *conn)
{
	Hstmt	= SQL_NULL_HSTMT;
	pConn	= conn;
	name	= query_name;
}

/*
 */
VDKODBCQuery::VDKODBCQuery(VDKString query_name, VDKODBCConnection *conn)
{
	VDKODBCQuery(query_name.c_str(), conn);
}

/*
 */
VDKODBCQuery::~VDKODBCQuery()
{
}

/*
 */
void
VDKODBCQuery::FreeStmt()
{
	SQLFreeHandle(SQL_HANDLE_STMT, Hstmt);
}

/*
 */
bool
VDKODBCQuery::ConnSanity()
{
	if (!pConn) {
		Err(VDKODBC_ERR_INVALID_CONN_POINTER, "Query <%s>", name.c_str());
		return false;
	}
	return true;
}

/*
 */
int
VDKODBCQuery::Prepare(const char *query, ...)
{
	va_list argptr;
	int		cnt;
	char	szBuffer[MAXPRINTFLEN];
		
	// Verify connection pointer validity
	if (!ConnSanity()) return LastNumError;
	// constructing the new string
	va_start(argptr, query);
	cnt = g_vsnprintf(szBuffer, MAXPRINTFLEN, query, argptr);
	va_end(argptr);
	if (cnt == EOF) return VDKODBC_MAX_ERR_BUF;
	// Allocate a statement handle
	if (!SQLTest(SQLAllocHandle(SQL_HANDLE_STMT, pConn->GetHdbc(), &Hstmt)))
		return LastNumError;
	// Prepare the query
	if (!SQLTest(SQLPrepare(Hstmt, (SQLCHAR*)szBuffer, strlen(szBuffer)))) {
		SQLFreeHandle(SQL_HANDLE_STMT,Hstmt);
		return LastNumError;
	}
	return SQL_SUCCESS;
}

/*
 */
int
VDKODBCQuery::Prepare(VDKString query)
{
	return Prepare("%s", query.c_str());
}

/*
 */
int
VDKODBCQuery::Exec()
{
	if (!ConnSanity()) return LastNumError;
	if (!SQLTest(SQLExecute(Hstmt), pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return LastNumError;
	return SQL_SUCCESS;
}

/*
 */
int
VDKODBCQuery::ExecDirect(const char *query, ...)
{
	va_list argptr;
	int		cnt;
	char	szBuffer[MAXPRINTFLEN];
		
	if (!ConnSanity()) return LastNumError;
	// constructing the new string
	va_start(argptr, query);
	cnt = g_vsnprintf(szBuffer, MAXPRINTFLEN, query, argptr);
	va_end(argptr);
	if (cnt == EOF) return VDKODBC_MAX_ERR_BUF;
	// Prepare the query
	if (!SQL_OK(Prepare("%s", szBuffer))) return LastNumError;
	// Execute it
	if (!SQL_OK(Exec())) return LastNumError;
	return SQL_SUCCESS;
}

/*
 */
int
VDKODBCQuery::ExecDirect(VDKString query)
{
	return ExecDirect("%s", query.c_str());
}

/*
 */
int
VDKODBCQuery::ColumnCount()
{
	SQLSMALLINT colcount;
	
	if (!ConnSanity()) return LastNumError;
	if (!SQLTest(SQLNumResultCols(Hstmt,&colcount)))
		return LastNumError;
	return (int)colcount;
}

/*
 */
int
VDKODBCQuery::RowCount()
{
	SQLINTEGER rowcount;
	
	if (!ConnSanity()) return LastNumError;
	if (!SQLTest(SQLRowCount(Hstmt,&rowcount))) return LastNumError;
	return (int)rowcount;
}

/*
 */
int
VDKODBCQuery::Fetch()
{
	int rslt;

	if (!ConnSanity()) return LastNumError;
	rslt = SQLFetch(Hstmt);
	if (!SQL_OK(rslt) && (rslt != SQL_NO_DATA)) SQLTest(rslt);
	return rslt;
}

/*
 */
void
VDKODBCQuery::Close()
{
	FreeStmt();
}

/*
 */
bool
VDKODBCQuery::Eof()
{
	int rslt = Fetch();
	return (!SQL_OK(rslt) || (rslt == SQL_NO_DATA));
}

/*
 */
int
VDKODBCQuery::GetData(unsigned int numcol, int coltype, char *buffer, int buflen)
{
	SQLINTEGER err;
	
	if (!ConnSanity()) return LastNumError;
	SQLGetData(Hstmt, numcol, coltype, buffer, buflen, &err);
	if (!SQLTest(err)) return LastNumError;
	return SQL_SUCCESS;
}

/*
 */
int
VDKODBCQuery::GetColDesc(const char *colname, ColDesc& coldesc)
{
	SQLSMALLINT i = 1;
	SQLINTEGER rslt = SQL_SUCCESS;

	if (!ConnSanity()) return LastNumError;
	int maxcol = ColumnCount();
	while (i <= maxcol) {
		rslt = SQLDescribeCol(Hstmt, i, coldesc.name, VDKODBC_MAXCOLNAMELEN+1, &coldesc.namelen,
							&coldesc.type, &coldesc.size, &coldesc.dec, &coldesc.nullable);
		if (!SQLTest(rslt, pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return rslt;
		coldesc.num = i;
		if (strcmp((const char*)coldesc.name, colname) == 0) return rslt;
		i++;
	}
	if ((i == maxcol) && strlen((const char*)coldesc.name) == 0) {
		rslt = VDKODBC_ERR_INVALID_COLUMN_NAME;
		Err(rslt, "Query <%s> column <%s>", name.c_str(), colname);
	}
	return rslt;
}

/*
 */
int
VDKODBCQuery::GetColDesc(VDKString colname, ColDesc& coldesc)
{
	return GetColDesc(colname.c_str(), coldesc);
}

/*
 */
int
VDKODBCQuery::GetColDesc(unsigned int colnum, ColDesc& coldesc)
{
	SQLSMALLINT i = colnum;
	SQLINTEGER rslt;

	if (!ConnSanity()) return LastNumError;
	unsigned int maxcol = ColumnCount();
	if (colnum > maxcol) {
		rslt = VDKODBC_ERR_INVALID_COLUMN_NUMBER;
		Err(rslt, "Query <%s> column <%d>", name.c_str(), colnum);
	}
	rslt = SQLDescribeCol(Hstmt, i, coldesc.name, VDKODBC_MAXCOLNAMELEN+1, &coldesc.namelen,
							&coldesc.type, &coldesc.size, &coldesc.dec, &coldesc.nullable);
	if (!SQLTest(rslt, pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return rslt;
	return rslt;
}

/*
*/
VDKString
VDKODBCQuery::GetColumnName(unsigned int colnum)
{
	SQLINTEGER	rslt;
	ColDesc		coldesc;
	VDKString	buf;

	rslt = GetColDesc(colnum, coldesc);
	if (!SQLTest(rslt, pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return buf;
	buf = (const char*)coldesc.name;
	return buf;
}

/*
 */
VDKString
VDKODBCQuery::GetFieldValue(const char *colname)
{
	SQLINTEGER	rslt;
	SQLCHAR		*szBuffer;
	ColDesc		coldesc;
	VDKString	value;
	
	if (!ConnSanity()) return value;
	rslt = GetColDesc(colname, coldesc);
	if (!SQLTest(rslt, pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return value;
/*
	printf("Column desc : \n\tNum=%d\n\tName=%s\n\tLen=%d\n"
			"\tType=%d\n\tSize=%ld\n\tDec=%d\n\tNull=%d\n",
			coldesc.num, coldesc.name, coldesc.namelen, coldesc.type, coldesc.size, coldesc.dec, coldesc.nullable);
*/
	if (coldesc.size == 0) {
		rslt = VDKODBC_ERR_NULL_FIELD_LENGTH;
		Err(rslt, "Query <%s> column <%s>", name.c_str(), colname);
		return value;
	}
	szBuffer = new SQLCHAR[coldesc.size+1];
	if (!szBuffer) {
		rslt = VDKODBC_ERR_MEMORY;
		Err(rslt, "Query <%s>", name.c_str());
		return value;
	}
	szBuffer[0] = 0;
	rslt = SQLGetData(Hstmt, coldesc.num, SQL_C_CHAR, szBuffer, coldesc.size+1, &rslt);
	if (!SQLTest(rslt)) return value;
	value = (char*)szBuffer;
	delete[] szBuffer;
	return value;
}

/*
 */
VDKString
VDKODBCQuery::GetFieldValue(VDKString colname)
{
	return GetFieldValue(colname.c_str());
}

/*
 */
VDKString
VDKODBCQuery::GetFieldValue(unsigned int colnum)
{
	SQLINTEGER	rslt;
	ColDesc		coldesc;
	
	if (!ConnSanity()) return VDKString("");
	rslt = GetColDesc(colnum, coldesc);
	if (!SQLTest(rslt, pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return VDKString("");
/*
	printf("Column desc : \n\tNum=%d\n\tName=%s\n\tLen=%d\n"
			"\tType=%d\n\tSize=%ld\n\tDec=%d\n\tNull=%d\n",
			coldesc.num, coldesc.name, coldesc.namelen, coldesc.type, coldesc.size, coldesc.dec, coldesc.nullable);
*/
//	printf("num=%d,val=%s\n", colnum, coldesc.name);
	return GetFieldValue((const char*)coldesc.name);
}


/*
 */
int
VDKODBCQuery::GetColumnType(const char *colname)
{
	SQLINTEGER	rslt;
	ColDesc		coldesc;

	rslt = GetColDesc(colname, coldesc);
	if (!SQLTest(rslt, pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return SQL_UNKNOWN_TYPE;
	return coldesc.type;
}

/*
 */
int
VDKODBCQuery::GetColumnType(VDKString colname)
{
	return GetColumnType(colname.c_str());
}

/*
 */
int
VDKODBCQuery::GetColumnType(unsigned int colnum)
{
	SQLINTEGER	rslt;
	ColDesc		coldesc;

	rslt = GetColDesc(colnum, coldesc);
	if (!SQLTest(rslt, pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return SQL_UNKNOWN_TYPE;
	return coldesc.type;
}

/*
 */
int
VDKODBCQuery::GetColumnSize(const char *colname)
{
	SQLINTEGER	rslt;
	ColDesc		coldesc;

	rslt = GetColDesc(colname, coldesc);
	if (!SQLTest(rslt, pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return 0;
	return coldesc.size;
}

/*
 */
int
VDKODBCQuery::GetColumnSize(VDKString colname)
{
	return GetColumnSize(colname.c_str());
}

/*
 */
int
VDKODBCQuery::GetColumnSize(unsigned int colnum)
{
	SQLINTEGER	rslt;
	ColDesc		coldesc;

	rslt = GetColDesc(colnum, coldesc);
	if (!SQLTest(rslt, pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return 0;
	return coldesc.size;
}

/*
 */
int
VDKODBCQuery::GetColumnDecDigits(const char *colname)
{
	SQLINTEGER	rslt;
	ColDesc		coldesc;

	rslt = GetColDesc(colname, coldesc);
	if (!SQLTest(rslt, pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return 0;
	return coldesc.dec;
}

/*
 */
int
VDKODBCQuery::GetColumnDecDigits(VDKString colname)
{
	return GetColumnDecDigits(colname.c_str());
}

/*
 */
int
VDKODBCQuery::GetColumnDecDigits(unsigned int colnum)
{
	SQLINTEGER	rslt;
	ColDesc		coldesc;

	rslt = GetColDesc(colnum, coldesc);
	if (!SQLTest(rslt, pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return 0;
	return coldesc.dec;
}

/*
 */
int
VDKODBCQuery::GetColumnNullable(const char *colname)
{
	SQLINTEGER	rslt;
	ColDesc		coldesc;

	rslt = GetColDesc(colname, coldesc);
	if (!SQLTest(rslt, pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return SQL_NULLABLE_UNKNOWN;
	return coldesc.nullable;
}

/*
 */
int
VDKODBCQuery::GetColumnNullable(VDKString colname)
{
	return GetColumnNullable(colname.c_str());
}

/*
 */
int
VDKODBCQuery::GetColumnNullable(unsigned int colnum)
{
	SQLINTEGER	rslt;
	ColDesc		coldesc;

	rslt = GetColDesc(colnum, coldesc);
	if (!SQLTest(rslt, pConn->GetHenv(), pConn->GetHdbc(), Hstmt)) return SQL_NULLABLE_UNKNOWN;
	return coldesc.nullable;
}

/*
*/
VDKString
VDKODBCQuery::GetColumnTypeName(int sqldatatype)
{
	SQLCHAR		buffer[VDKODBC_MAXCOLNAMELEN] = "";
	VDKString	Result;
	SQLINTEGER	len;
	SQLHSTMT	tmpHstmt;

	if (!ConnSanity()) return Result;
	// Transform sqldatatype value from ODBC 3 to ODBC 2 compliance
	// Not reliable - plm - 2003/04/07, waiting for driver progress...
	switch (sqldatatype) {
	  case SQL_TYPE_DATE:
	  case SQL_DATE:
	    Result = "date";
		return Result;
	  case SQL_TYPE_TIME:
	  case SQL_TIME:
	    Result = "time";
		return Result;
	  case SQL_TYPE_TIMESTAMP:
	  case SQL_TIMESTAMP:
	    Result = "timestamp";
		return Result;
	}
	if (!SQLTest(SQLAllocHandle(SQL_HANDLE_STMT, pConn->GetHdbc(), &tmpHstmt))) return Result;
	if (SQLTest(SQLGetTypeInfo(tmpHstmt, sqldatatype)) && // Prepare the query
		(SQLFetch(tmpHstmt) != SQL_NO_DATA) && // Fetch the data
		(SQLTest(SQLGetData(tmpHstmt, 1, SQL_C_CHAR, buffer, VDKODBC_MAXCOLNAMELEN, &len))))
		  Result = (const char*)buffer;
	SQLFreeHandle(SQL_HANDLE_STMT, tmpHstmt);
	return Result;
}
